#!/usr/bin/env python3
"""CLI Command utilities."""


from multiprocessing import Process
from time import sleep
from typing import Optional

import click


class Printer:
    """Custom Styled Printer class."""

    def __init__(self) -> None:
        """Printer class initialisation method."""
        self.left_part = "="
        self.center_part = "*"
        self.right_part = "="
        self.max_animation_width = 5
        self.wip_content_length = 0
        self.process: Optional[Process] = None

    def loop_animation(self, position: int, last_postition: int) -> int:
        """Loop animation logic.

        Args:
            position (int): Current position of moving part
            last_postition (int): Final position of moving part

        Returns:
            int: Calculated position of moving part
        """
        if position < last_postition:
            position += 1
        else:
            position = 1
        return position

    # def bounce_animation(
    #     self, direction, position, last_postition
    # ) -> Tuple[str, str]:
    #     if direction:
    #         if position < last_postition:
    #             position += 1
    #         else:
    #             direction = not direction
    #             position -= 1
    #     else:
    #         if position > 1:
    #             position -= 1
    #         else:
    #             direction = not direction
    #             position += 1

    #     return direction, position

    def static_arrow(self, content: str) -> None:
        """Non-animated printing method for first line.

        Args:
            content (str): Content to print
        """
        click.secho("\r====> ", nl=False, bold=True, fg="magenta")
        click.secho(content, fg="green")

    def animate(self, content: str, speed: float = 0.2) -> None:
        """Add animation in front of content using multithreading.

        Args:
            content (str): Content to print
            speed (float, optional): Time intervel between each animation frame. Defaults to 0.2.
        """
        if self.process is not None:
            pos = 1
            c_len = len(self.center_part)
            last_pos = self.max_animation_width - c_len + 1
            while True:
                l_len = pos - 1
                r_len = self.max_animation_width - c_len - l_len
                left = self.left_part * l_len
                right = self.right_part * r_len
                arrow = f"{left}{self.center_part}{right}"
                click.secho(
                    f"\r{arrow} ", nl=False, bold=True, fg="bright_cyan"
                )
                click.secho(content, nl=False, fg="bright_yellow")
                sleep(speed)
                pos = self.loop_animation(pos, last_pos)

    def stop_animation(self) -> None:
        """Stop the current animation in progress."""
        if self.process is not None:
            self.process.terminate()
            self.process = None

    def working(self, content: str, animate: bool = True) -> None:
        """Print animated content with WIP context.

        Args:
            content (str): Content to print
            animate (bool, optional): [description]. Defaults to True.
        """
        self.wip_content_length = len(content)
        if animate:
            self.process = Process(target=self.animate, args=(content,))
            self.process.start()
        else:
            self.static_arrow(content)

    def done(self, content: str, url: str = "") -> None:
        """Print completed content.

        Args:
            content (str): Content to print
            url (str, optional): URL to print and launch in browser. Defaults to "".
        """
        correction_length = self.wip_content_length - len(content)
        empty_string = ""
        if correction_length > 0:
            empty_string = " " * correction_length
        self.stop_animation()
        click.secho("\r<===> ", nl=False, fg="magenta")
        if not url:
            click.secho(f"{content}{empty_string}", fg="green")
        else:
            click.secho(f"{content} ", nl=False, fg="green")
            click.secho(url, nl=False, fg="bright_blue", underline=True)
            click.echo(empty_string)
            click.launch(url)



#!/usr/bin/env python3
"""Cloudflare Worker module to deploy Root certificate."""

from pathlib import Path
from typing import Optional, Union

from CloudflareAPI import Cloudflare
from CloudflareAPI.api import Worker as CFWorker
from CloudflareAPI.exceptions import CFError


class Worker:
    """Cloudflare Worker class which handle the deployment of Root certificate."""

    WORKER_NS_NAME_KEY = "CA_CERT_STORE"
    WORKER_TITLE_KEY = "CA_TITLE"
    WORKER_FINGERPRINT_KEY = "ROOT_CA_FINGERPRINT"
    WORKER_CA_URL_KEY = "ROOT_CA_URL"

    def __init__(self, token: Optional[str] = None) -> None:
        """Initialise the Cloudflare API.

        Args:
            token (str): Optional argument to pass Cloudflare API Token
        """
        self.api = Cloudflare(token=token)
        self.metadata: Optional[CFWorker.Metadata] = None

    def store(self, web_title: str, fingerprint: str, ca_url: str) -> None:
        """Store data to be published.

        Args:
            web_title (str): Title of the Worker
            fingerprint (str): Fingerprint to attach to the worker
            ca_url (str): CA URL to attach to the worker
        """
        self.title, self.fingerprint, self.url = (
            web_title,
            fingerprint,
            ca_url,
        )

    def get_metadata(self, namespace) -> None:
        """Generate for the Cloudflare worker.

        Args:
            namespace (CloudflareAPI.dataclass.Namespace): Namespace instance of Cloudflare API.
        """
        self.metadata = self.api.worker.Metadata()
        self.metadata.add_variable(self.WORKER_TITLE_KEY, self.title)
        self.metadata.add_variable(
            self.WORKER_FINGERPRINT_KEY, self.fingerprint
        )
        self.metadata.add_variable(self.WORKER_CA_URL_KEY, self.url)
        self.metadata.add_binding(self.WORKER_NS_NAME_KEY, namespace.id)

    def loadCA(self, rootCA: Union[str, Path]) -> None:
        """Load the CA certificate and write to the Cloudflare KV Namespace.

        Args:
            rootCA (str): Root Certificate file location
        """
        rootca: Optional[Union[bytes, str]] = None
        rootCA_file = Path(rootCA) if not isinstance(rootCA, Path) else rootCA
        try:
            namespace = self.api.store.get_ns(self.WORKER_NS_NAME_KEY)
        except CFError:
            namespace = self.api.store.create(self.WORKER_NS_NAME_KEY)
        try:
            rootca = rootCA_file.read_text()
            namespace.write("root_ca_format", "pem")
        except UnicodeDecodeError:
            rootca = rootCA_file.read_bytes()
            namespace.write("root_ca_format", "der")
        namespace.write("root_ca", rootca)
        self.get_metadata(namespace)

    def deploy(self, name: str, file: Union[str, Path]) -> str:
        """Deploy the worker in to Cloudflare Edge network.

        Args:
            name (str): Name of worker. This name will be reflected in the worker url.
            file (str): Javascript file of the worker to deploy

        Returns:
            str: Worker url which is deployed in to CLoudflare Edge network
        """
        if self.metadata is not None:
            worker_name = name.strip().lower()
            worker_file = (
                Path(file).resolve(strict=True)
                if not isinstance(file, Path)
                else file
            )
            if self.api.worker.upload(
                name=worker_name,
                file=worker_file,
                metadata=self.metadata,
            ):
                if self.api.worker.deploy(worker_name):
                    subdomain = self.api.worker.subdomain.get()
                    return f"https://{worker_name}.{subdomain}.workers.dev"

            raise CFError("Deployment failed")

        raise CFError("Metadata not found")



#!/usr/bin/env python3
"""CLI Command to deploy cloudflare worker."""

from pathlib import Path

import click

from scapy import Worker

from .utils import Printer

HOME = Path.home()


@click.command()
@click.option(
    "-t",
    "--token",
    envvar="CF_TOKEN",
    default=None,
    type=str,
    help="Cloudflare API Token to access workers",
)
@click.option(
    "-w",
    "--worker",
    envvar="WORKER",
    default="ca",
    type=str,
    help="Name of the worker",
)
@click.option(
    "-j",
    "--js",
    envvar="WORKER_FILE",
    default=Path("./build/index.js"),
    type=click.Path(exists=True),
    help="Worker file location",
)
@click.option(
    "-n",
    "--name",
    envvar="WEB_TITLE",
    type=str,
    help="CA Server's Name",
)
@click.option(
    "-f",
    "--fingerprint",
    envvar="FINGERPRINT",
    type=str,
    help="Root CA Certificate's fingerprint",
)
@click.option(
    "-u",
    "--ca-url",
    envvar="CA_URL",
    default="https://stepca.local",
    type=str,
    help="CA Server's URL",
)
@click.option(
    "-r",
    "--root",
    envvar="ROOT_CERT",
    default=(HOME / ".step/certs/root_ca.crt"),
    type=click.Path(exists=True),
    help="CA Root Certificate file in PEM or DER format",
)
def deploy(
    token: str,
    worker: str,
    js: Path,
    name: str,
    fingerprint: str,
    ca_url: str,
    root: Path,
):
    """Deploy a cloudflare worker to publish CA Certificate."""
    printer = Printer()
    printer.working("Initialising deployment process", animate=False)
    deployer = Worker(token)
    printer.done("Worker instance is created")
    deployer.store(
        web_title=name,
        fingerprint=fingerprint,
        ca_url=ca_url,
    )
    printer.done("Updated worker with given data")
    printer.working("Uploding Root Certificate")
    deployer.loadCA(root)
    printer.done("Root Certificate is uploaded")
    printer.working("Deploying the worker")
    deployed_url = deployer.deploy(worker, js)
    printer.done("Deployed worker:", deployed_url)



"""Scapy Manager package setup file."""

import re
import pathlib

from setuptools import find_packages, setup


def main():
    """Scapy Manager's main setup method."""
    HERE = pathlib.Path(__file__).parent
    version_file = HERE / "scapy/__version__.py"
    version_file = version_file.resolve(strict=True).read_text()
    VERSION = re.compile(r"#v(.+)").search(version_file).group(1)

    setup(
        name="scapy-man",
        version=VERSION,
        description="Step CA Manager using Python",
        long_description=(HERE / "README.md").read_text(),
        long_description_content_type="text/markdown",
        author="Nikhil John",
        author_email="nikhiljohn1010@gmail.com",
        url="https://github.com/nikhiljohn10/scapy",
        license="MIT",
        packages=find_packages(),
        py_modules=["scapy"],
        install_requires=["cloudflare-api>=2.0.4", "Click"],
        entry_points={
            "console_scripts": [
                "scapy = scapy.cli:cli",
            ],
        },
        keywords=["step-ca", "cloudflare-api", "workers", "certificate"],
        classifiers=[
            "Development Status :: 4 - Beta",
            "Intended Audience :: Developers",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
        ],
    )


if __name__ == "__main__":
    main()
